# distutils: language=c++
# -*- coding: utf-8 -*-

'''
    Utility functions for validation
'''

def to_type_names_string(types, conjunction='or'):
    if isinstance(types, (list, tuple)):
        length = len(types)
        if length == 0:
            raise ValueError("Expected non-empty list or tuple, but given empty: {}".format(types))
        elif length == 1:
            return to_type_names_string(types[0])
        else:
            type_names = list(map(to_type_names_string, types))
            if length == 2:
                return '{} {} {}'.format(type_names[0], conjunction, type_names[1])
            else:
                left = str.join(', ', type_names[0:-1])
                right = type_names[-1]
                return '{} {} {}'.format(left, conjunction, right)
    else:
        elem = types
        if isinstance(elem, type):
            return elem.__name__
        else:
            raise TypeError("Expected type object, but given non-type object: {}".format(elem))

def check_argument_type(val, name, expected_types):
    if isinstance(expected_types, list):
        expected_types = tuple(expected_types)
    if isinstance(val, expected_types):
        # OK
        return True
    else:
        type_names = to_type_names_string(expected_types)
        tmp_error = 'Invalid type of argument `{}` is given: expected {}, but given {}'
        raise TypeError(tmp_error.format(name, type_names, type(val).__name__))




from libcpp cimport bool

cdef class StringEnumerator:
    cdef dict dict_str2id
    cdef list list_id2str

    cpdef bool append(self, str string)
    cpdef long str2id(self, str string)
    cpdef str id2str(self, long number)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Standard libraries
import argparse
import os
import sys
import time

# Local libraries
from lpu.common import logging

logger = logging.getColorLogger(__name__)

def waitFile(filepath, interval=1, timeout=0):
    if interval < 0:
        interval = 1
    if not os.path.exists(filepath):
        logger.info("Waiting for file: %s" % filepath)
        firstTime = time.time()
    while not os.path.exists(filepath):
        time.sleep(interval)
        elapsed = time.time() - firstTime
        if timeout > 0 and elapsed > timeout:
                logger.error("Waiting file (%s) was timed out (%s seconds)" % (filepath, timeout))
    logger.info("File exists: %s" % filepath)
    return True

def waitFiles(filepaths, interval=1, timeout=0):
    if type(filepaths) == str:
        filepaths = [filepaths]
    for path in filepaths:
        waitFile(path, interval, timeout)

def cmdWaitFiles(args):
    parser = argparse.ArgumentParser(description='Wait until file will be found')
    parser.add_argument('filepaths', metavar='filepath', nargs='+', help='file path for waiting')
    parser.add_argument('--quiet', '-q', action='store_true', help='quiet mode')
    parser.add_argument('--debug', '-D', action='store_true', help='debug mode')
    parser.add_argument('--interval', '-i', default=1, type=float, help='interval for next trial (default: %(default)s seconds)')
    parser.add_argument('--timeout', '-t', default=0, type=float, help='time limit in waiting file')
    parsed = parser.parse_args(args)

    with logging.using_config(logger) as c:
        if parsed.debug:
            c.set_debug(True)
            c.set_quiet(False)
        if parsed.quiet:
            c.set_quiet(True)
            c.set_debug(False)
        logger.debug(parsed)
        #waitFiles(**vars(parsed))
        waitFiles(parsed.filepaths, parsed.interval, parsed.timeout)

def main():
    cmdWaitFiles(sys.argv[1:])

if __name__ == '__main__':
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''phrase/rule table filtering function'''

# Standard libraries
import argparse
import os
import pprint
import re
import sys
import subprocess

# Local libraries
from lpu.common import files
from lpu.common import progress
from lpu.smt.trans_models import records

def matchRules(rec, rules):
    for rule in rules:
        expr = rule
        expr = re.sub('c\.c', str(rec.counts.cooc),  expr)
        expr = re.sub('c\.s', str(rec.counts.src), expr)
        expr = re.sub('c\.t', str(rec.counts.trg), expr)
        #debug.log(expr)
        if eval(expr):
            #print("MATCH")
            pass
        else:
            #print("MISMATCH")
            return False
    return True

def saveRecords(saveFile, records, nbest):
    pairs = []
    if nbest > 0:
        for i, rec in enumerate(records):
            pairs.append( (rec.features['egfp'],i) )
        pairs.sort(reverse = True)
        newRecords = []
        for _, i in pairs[:nbest]:
            newRecords.append( records[i] )
        records = newRecords
    for rec in records:
        saveFile.write(rec.to_str())
        saveFile.write("\n")

#def filterTable(srcFile, saveFile, rules, nbest, progress = False, RecordClass = records.MosesRecord):
def filterTable(srcFile, saveFile, rules, nbest, RecordClass = records.MosesRecord):
    #if type(srcFile) == str:
    #    #srcFile = files.open(srcFile)
    #    srcFile = files.open(srcFile, 'rt')
    if type(saveFile) == str:
        saveFile = files.open(saveFile, 'wt')
    #if progress:
    #    srcFile = progress.view(srcFile)
    #srcFile = progress.view(srcFile, 'processing')
    records = []
    lastSrc = ''
    for line in progress.view(srcFile, 'processing'):
        rec = RecordClass(line)
        if rec.src != lastSrc:
            lastSrc = rec.src
            if records:
                saveRecords(saveFile, records, nbest)
                records = []
        if matchRules(rec, rules):
            records.append( rec )
#          output.write( rec.to_str() )
    if records:
        saveRecords(saveFile, records, nbest)
    saveFile.close()

#def filterMosesTable(srcFile, saveFile, rules, nbest, progress = False):
def filterMosesTable(srcFile, saveFile, rules, nbest):
    #filterTable(srcFile, saveFile, rules, nbest, progress = progress, RecordClass = records.MosesRecord)
    filterTable(srcFile, saveFile, rules, nbest, RecordClass = records.MosesRecord)

#def main():
#    epilog = '''
#each rule should be as '{varname} {<,<=,==,>=,>} {value}'
#varnames:
#    c.s : source count
#    c.t : target count
#    c.c : co-occurrence count
#example:
#    %s model/phrase-table.gz model/filtered-table.gz 'c.c > 1'
#    ''' % sys.argv[0]
#    parser = argparse.ArgumentParser(
#        formatter_class=argparse.RawDescriptionHelpFormatter,
#        description='filter moses phrase-table by supplied rules',
#        epilog = epilog,
#    )
#    parser.add_argument('srcFile',  help='file path to load phrase-table')
#    parser.add_argument('saveFile', help='file path to save phrase-table')
#    parser.add_argument('rules', metavar='rule', nargs='+', help='filtering rule to save record')
#    parser.add_argument('--nbest', '-n', type=int, default=0,
#                        help='target variation limit for one source')
#    parser.add_argument('--progress', '-p', action='store_true',
#                        help='show progress bar (pv command should be installed')
#    args = vars(parser.parse_args())
#    #print(args)
#    filterMosesTable(**args)

def filterTravatarTable(srcFile, saveFile, rules, nbest, progress = False):
    #filterTable(srcFile, saveFile, rules, nbest, progress = progress, RecordClass = records.TravatarRecord)
    filterTable(srcFile, saveFile, rules, nbest, RecordClass = records.TravatarRecord)

def main():
    epilog = '''
each rule should be as '{varname} {<,<=,==,>=,>} {value}'
varnames:
    c.s : source count
    c.t : target count
    c.c : co-occurrence count
example:
    %s model/rule-table.gz model/filtered-table.gz 'c.c > 1'
    ''' % sys.argv[0]
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description='filter travatar rule-table by supplied rules',
        epilog = epilog,
    )
    parser.add_argument('srcFile',  help='file path to load rule-table')
    parser.add_argument('saveFile', help='file path to save rule-table')
    parser.add_argument('rules', metavar='rule', nargs='+', help='filtering rule to save record')
    parser.add_argument('--nbest', '-n', type=int, default=0,
                        help='target variation limit for one source')
    parser.add_argument('--progress', '-p', action='store_true',
                        help='show progress bar (pv command should be installed')
    args = vars(parser.parse_args())
    #print(args)
    filterTravatarTable(**args)

if __name__ == '__main__':
    main()

